/*
 *   Copyright 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago4Q.

    Spago4Q is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published 
    by the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spago4Q is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package it.eng;

import java.math.BigDecimal;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.ArrayList;

import net.sf.jasperreports.engine.JRDefaultScriptlet;
import net.sf.jasperreports.engine.JRScriptletException;
import net.sf.jasperreports.renderers.JCommonDrawableRenderer;

import org.jfree.chart.JFreeChart;

public class BugsPlannedVsActualEffort extends JRDefaultScriptlet{

	private Connection dbConnection;
	private PreparedStatement stmt = null;
	private ResultSet rs = null;
	private ReportUtilities ru;

    private String parArea="";
    private String parProject="";
    private String parYear="";
    private String parMonth="";
    private String parCustomer="";
    private String parMethod="";
	
	private final int idKPI = 16;


	public void afterReportInit() throws JRScriptletException {

		try {
			// get db connection
			dbConnection = (Connection) this.getParameterValue("REPORT_CONNECTION");

			ru=new ReportUtilities(dbConnection); 

			// set idKPI


			// get meterChart value
			BigDecimal value = this.calculateValue(idKPI);

			// calculate kpi threshold
			int thresholdType = 0;
			int idUser = 0;
			int idThreshold = ru.calculateThreshold(thresholdType, idUser,idKPI);

			// get meterChart title
			String title = ru.getKPITitle(idKPI);

			ArrayList<Pair> subs=ru.getSubtitles(parArea, parProject, parCustomer, parMethod);
			
			Pair year=null;
			Pair month=null;
			if(parYear.equalsIgnoreCase("0"))year=new Pair("Year","All Years");
			else year=new Pair("Year", parYear);  
			
			if(parMonth.equalsIgnoreCase("0"))month=new Pair("Month","All Monthes");
			else month=new Pair("Month", parMonth);  				
			
			subs.add(year);
			subs.add(month);
			
			// create meter chart
			JFreeChart meterchart = ru.createMeterChart(value, title, idThreshold,subs);
			this.setVariableValue("Chart_meter", new JCommonDrawableRenderer(
					meterchart));

		} catch (Exception e) {
			throw new JRScriptletException(e);
		}
	}

	private BigDecimal calculateValue(int pIdKPI) throws JRScriptletException {
		BigDecimal plannedEff = new BigDecimal(0);
		BigDecimal actualeff = new BigDecimal(0);
		BigDecimal totalValue = new BigDecimal(0);
		
		try {
			
			
			parMonth = (String) this.getParameterValue("MonthPar");
			parYear = (String) this.getParameterValue("YearPar");
			parArea = (String) this.getParameterValue("AreaPar");
			parProject = (String) this.getParameterValue("ProjectPar");
			parCustomer= (String) this.getParameterValue("CustomerPar");
			parMethod= (String) this.getParameterValue("MethodPar");
			
			String query = "select sum(planned_effort)as plannedEff, sum(actual_effort) as actualeff" +
					" from ft_bug as ft, dt_project_structure_level3 as p3," +
					" dt_project_structure_level2 as p2, dt_project_structure_level1 as p1," +
					" dt_project as p, dt_time as dt" +
					" where" +
					" ft.id_project_structure_level3 = p3.id_level3" +
					" and p3.id_level2 = p2.id_level2" +
					" and p2.id_level1 = p1.id_level1" +
					" and p1.id_level1 = p.id_project" +
					" and ft.id_creation_time = dt.id_time";


			String queryPeriod = ru.buildPeriod(parYear, parMonth);
			String queryProfile = ru.buildProfile(parArea, parProject,"p","p");
			String queryCustomer = ru.buildCustomerMethod(parCustomer, parMethod,"p","p");

			stmt = dbConnection.prepareStatement(query + queryPeriod + queryProfile);
			System.out.println("query: " +query + queryPeriod + queryProfile + queryCustomer);

			stmt.execute();

			rs = stmt.executeQuery();
			if (rs.next()) {
				plannedEff = rs.getBigDecimal("plannedEff");
				actualeff = rs.getBigDecimal("actualEff");
			}
		} catch (Exception e) {
			throw new JRScriptletException(e);
		}
		
		totalValue=new BigDecimal(actualeff.intValue()-plannedEff.intValue());
		
		if (totalValue == null) totalValue = BigDecimal.valueOf(0);
		return totalValue;
	}


}

